//CW3DMilkshapeModel
//Brent Gunning
//Copyright 2002 All Rights Reserved

//loads and renders a static milkshape model. It has the functionality
//of bounding sphere's a boxes so we use it for loading the levels and
//other static meshes.

//define the file
#ifndef CW3DMILKSHAPEMODEL_H
#define CW3DMILKSHAPEMODEL_H

#include "MilkshapeLoader.h"
#include "MilkshapeFormat.h"
#include "ModelBoundings.h"
#include "VertexFormats.h"
#include "FileUtils.h"
#include "CW3DFrustum.h"

//define for the vertex structure
#define MS3DMODELVERTEX VERTEX_XYZ_NRM_TEX
#define D3DFVF_MS3DMODELVERTEX D3DFVF_VERTEX_XYZ_NRM_TEX

//model group structure
//holds the data for a milkshape model group
struct sModelGroup
{
	int NumFaces;
	D3DMATERIAL8 Material;
	IDirect3DTexture8* Texture;
	IDirect3DVertexBuffer8* VertexBuffer;
	sBoundingBox BoundingBox;
	sBoundingSphere BoundingSphere;
};

//the actual milkshape model class
//loads and renders a static milkshape model
class CW3DMilkshapeModel
{
public:
	//constructor and deconstructor
	//sets default values and releases memory
	CW3DMilkshapeModel();
	~CW3DMilkshapeModel();

	//this will load the model into the model group structures
	//and set everything up to render the static mesh
	HRESULT Initialize(char* strFilename,char* strRepathLocation,
		IDirect3DDevice8* pDevice);

	//this renders the model. it loops through all the model groups, set the texture
	//and material, checks if it's within the frustum if any, and renders the buffer
	HRESULT Render(CW3DFrustum* pFrustum,bool bCheckBoundingBox,
		bool bCheckBoundingSphere,IDirect3DDevice8* pDevice);

	//releases the memory for the class
	HRESULT Destroy();
protected:
	//model groups
	sModelGroup* m_pGroups;
	int m_NumGroups;

	//error variable
	HRESULT hr;
};

#endif