//CW3DParticleSystem.h
//Brent Gunning
//Copyright 2002 All Rights Reserved

//encapsulates the functionality of a particle system
//I tried to optomize it using triangles instead of quads
//and using dynamic vertex buffers but I'm sure there's more 
//optomizations. I could have used a vshader to transform
//the vertices into billboards but it makes the code more 
//confusing with vshader code. For this demo, I tried to 
//eliminate all vshader code.

//define the file
#ifndef CW3DPARTICLESYSTEM_H
#define CW3DPARTICLESYSTEM_H

//include d3d files
#include "d3d8.h"
#include "d3dx8.h"

//include the vertex formats used in the system
#include "VertexFormats.h"

//define for the particle vertex
#define PARTICLEVERTEX VERTEX_XYZ_NRM_DIF_TEX
#define D3DFVF_PARTICLEVERTEX D3DFVF_VERTEX_XYZ_NRM_DIF_TEX

//the particle structure.
//i could have put it in a 
//class but there wouldn't 
//be too much point to that
struct sParticle
{
	D3DXVECTOR3 pos;//position
	D3DXVECTOR3 vel;//velocity
	int life;		//current age
	D3DCOLOR color;	//color
	float weight;	//weight
	bool bAlive;	//if it's alive or not
};

//particle system properties structure
//it's used to create the system.
struct sParticleSysProps
{
	D3DXVECTOR3 vecInitPos;//initial position of creation
	D3DXVECTOR3 vecPosRand;//randomness of the position
	D3DXVECTOR3 vecInitVel;//initial velocity of creation
	D3DXVECTOR3 vecVelRand;//randomness of the velocities
	float fWeight;//weight of the particles
	float fWeightRand;//randomness of the weight
	int iLifetime;//total life of a particle
	int iLifetimeRand;//lifetime randomness
	D3DXCOLOR cStartColor;//starting color
	D3DXCOLOR cEndColor;//interpolated to this color
	int iNumParticles;//number of particles in the system
	D3DXVECTOR3 vecWind;//wind variable
	float fXSize,fYSize;//size of the particles
	bool bRemakeParticles;//if it remakes the particles or not
};

//the particle system class
//it was created so that you could derive from
//it if you really wanted to although I don't
//do so
class CW3DParticleSystem
{
public:
	//constructor/deconstructor
	//they just release the memory and set 0's
	CW3DParticleSystem();
	virtual ~CW3DParticleSystem();

	//will create the particles baesd on the particle
	//system properties structure given.
	virtual HRESULT Initialize(sParticleSysProps psp,IDirect3DDevice8* pDevice);

	//will update the particles. updates various things
	//such as the color, positon, etc. to create the billboards
	//from them
	virtual HRESULT Process();

	//releases the memory for the class
	virtual HRESULT Destroy();

	//it will render the particles using a vertex buffer.
	virtual HRESULT Render(IDirect3DDevice8* pDevice,D3DXMATRIX matWorld);

	//function to set the particle properties structure. certain values cannot
	//be changed but the color and stuff is allowed to be changed
	void SetParticleSystemProperties(sParticleSysProps psp) {m_psp=psp;}
protected:
	//function to generate a random number
	//used with the randomness variables
	float RandomFloat(float low,float hi);

protected:
	//particle properties structure
	sParticleSysProps m_psp;

	//vertex buffer for particles
	IDirect3DVertexBuffer8* m_VB;

	//the particles
	sParticle* m_pParticles;

	//error variable
	HRESULT hr;
};

#endif
