//MilkshapeLoader.h
//Brent Gunning
//Copyright 2002 All Rights Reserved

//using special structures and based on the milkshape format, it
//loads a milkshape model into an easy to access structure. This
//structure is used in the CW3DMilkshapeModel class and other 
//game classes

#ifndef MILKSHAPELOADER_H
#define MILKSHAPELOADER_H

//include files
#include "d3d8.h"
#include "d3dx8.h"
#include "MilkshapeFormat.h"
#include "FileUtils.h"
#include "stdio.h"

//basic structures
struct sVertex
{
	D3DXVECTOR3 vecPos;
	int BoneIndex;
};
struct sFace
{
	int VertexIndices[3];
	D3DXVECTOR3 VertexNormals[3];
	float tu[3],tv[3];
};
struct sGroup
{
	int NumFaces;
	int* pFaceIndices;
	int MaterialIndex;
};
struct sMaterial
{
	D3DMATERIAL8 Material;
	char* TextureFilename;
};

//structure for which all the previous structures
//are wrapped up into. This is the returned structure
//for the milkshape3d loading function
struct sMilkshapeData
{
	int NumVertices;
	int NumFaces;
	int NumGroups;
	int NumMaterials;
	sVertex* pVertices;
	sFace* pFaces;
	sGroup* pGroups;
	sMaterial* pMaterials;

	//constructor/deconstructor
	sMilkshapeData()
	{
		NumVertices=NumFaces=NumGroups=NumMaterials=0;
		pVertices=0;
		pFaces=0;
		pGroups=0;
		pMaterials=0;
	}
	~sMilkshapeData()
	{
		if(pVertices)
			delete[] pVertices;
		if(pFaces)
			delete[] pFaces;
		if(pGroups)
		{
			for(int i=0;i<NumGroups;i++)
			{
				if(pGroups[i].pFaceIndices)
					delete[] pGroups[i].pFaceIndices;
			}
			delete[] pGroups;
		}
		if(pMaterials)
		{
			for(int i=0;i<NumMaterials;i++)
			{
				if(pMaterials[i].TextureFilename)
					delete[] pMaterials[i].TextureFilename;
			}
			delete[] pMaterials;
		}
	}
};

//this is the actual function to load the bones into the milkshape data structure
//Note: It loads the file in binary format. It cannot load a ascii formatted 
//milkshape file
sMilkshapeData* LoadMilkshapeModel(char* strFilename);

#endif
