//CW3DInput.cpp
//Brent Gunning
//Copyright 2002 All Rights Reserved

//include header file
#include "../headers/CW3DInput.h"

//constructor/deconstructor
//sets defaults and releases memory
CW3DInput::CW3DInput()
{
	//set 0's
	m_Mouse=0;
	m_Keyboard=0;
	m_Input=0;
	hr=0;
	m_hWnd=0;
}
CW3DInput::~CW3DInput()
{
	//destroy the input class's memory
	Destroy();
}
//initializes direct input and its devices
//it returns a failed value if it failed
HRESULT CW3DInput::Initialize(HWND hWnd)
{
	//first release any memory that was already created
	Destroy();

	//set hwnd member variable for storage
	m_hWnd=hWnd;

	//create the direct input 8 object
	if(FAILED(hr=DirectInput8Create(GetModuleHandle(0),DIRECTINPUT_VERSION,IID_IDirectInput8,
									(void**)&m_Input,0)))
		return(hr);

	//now create the devices
	if(FAILED(hr=m_Input->CreateDevice(GUID_SysKeyboard,&m_Keyboard,0)))
	{
		Destroy();
		return(hr);
	}
	if(FAILED(hr=m_Input->CreateDevice(GUID_SysMouse,&m_Mouse,0)))
	{
		Destroy();
		return(hr);
	}

	//now we have to set the format for which the data will be in. lucky for us,
	//direct input has premade formats for the common devices
	if(FAILED(hr=m_Keyboard->SetDataFormat(&c_dfDIKeyboard)))
	{
		Destroy();
		return(hr);
	}
	if(FAILED(hr=m_Mouse->SetDataFormat(&c_dfDIMouse)))
	{
		Destroy();
		return(hr);
	}

	//now we have to tell the devices how it should respond and it's cooperativeness
	//with other windows
	if(FAILED(hr=m_Keyboard->SetCooperativeLevel(hWnd,DISCL_FOREGROUND|DISCL_EXCLUSIVE)))
	{
		Destroy();
		return(hr);
	}
	if(FAILED(hr=m_Mouse->SetCooperativeLevel(hWnd,DISCL_FOREGROUND|DISCL_EXCLUSIVE)))
	{
		Destroy();
		return(hr);
	}

	//now we need to acquire the actual hardware for reading from
	if(FAILED(hr=m_Keyboard->Acquire()))
	{
		Destroy();
		return(hr);
	}
	if(FAILED(hr=m_Mouse->Acquire()))
	{
		Destroy();
		return(hr);
	}
	
	//return success
	return(S_OK);
}
//it retrieves the data from the devices and
//returns it as pointers
HRESULT CW3DInput::Process(BYTE* pKeyboardState[256],DIMOUSESTATE* pMouseState)
{
	//get the current states of each
	//also test if the device was lost
	if(FAILED(hr=m_Keyboard->GetDeviceState(sizeof(BYTE)*256,pKeyboardState)))
	{
		//check if the input was lost
		if(hr=DIERR_INPUTLOST)
		{
			if(FAILED(hr=Destroy()))
				return(hr);
			if(FAILED(hr=Initialize(m_hWnd)))
				return(hr);
			return(S_OK);
		}
		return(hr);
	}
	if(FAILED(hr=m_Mouse->GetDeviceState(sizeof(DIMOUSESTATE),pMouseState)))
		return(hr);

	//return success
	return(S_OK);
}
//releases the memory for this class
HRESULT CW3DInput::Destroy()
{
	//release the mouse and keyboard devices
	//we need to unacquire them though
	if(m_Mouse)
	{
		if(FAILED(hr=m_Mouse->Unacquire()))
			return(hr);
		if(FAILED(hr=m_Mouse->Release()))
			return(hr);
		m_Mouse=0;
	}
	if(m_Keyboard)
	{
		if(FAILED(hr=m_Keyboard->Unacquire()))
			return(hr);
		if(FAILED(hr=m_Keyboard->Release()))
			return(hr);
		m_Keyboard=0;
	}
	//after releases the devices, releases the main input object. 
	//we release the devices first because it's a good habit to 
	//release objects in the opposite order they were created
	if(m_Input)
	{
		if(FAILED(hr=m_Input->Release()))
			return(hr);
		m_Input=0;
	}
	//return success
	return(S_OK);
}