//CW3DMD2Model.cpp
//Brent Gunning
//Copyright 2002 All Rights Reserved

#include "../headers/CW3DMD2Model.h"

//constructor/deconstruct
//sets defaults and releases memory
CW3DMD2Model::CW3DMD2Model()
{
	m_pFrames=0;
	m_NumFrames=0;
}
CW3DMD2Model::~CW3DMD2Model()
{
	Destroy();
}
//loads the model and initializes the frames
HRESULT CW3DMD2Model::Initialize(char* Filename,IDirect3DDevice8* pDevice)
{
	//destroy the model if it was already initialized
	Destroy();

	BYTE buffer[8320];

	//first open the file and prepare to load the data
	//we open the file in binary format because that's
	//the format which md2's are stored
	FILE* file=fopen(Filename,"rb");

	//check if the file opened correctly
	if(file==0)
		return(E_FAIL);

	//read in the md2 header
	sMD2Header header;
	fread(&header,1,sizeof(sMD2Header),file);

	//make sure the version is 8 or it's not the correct file
	if(header.Version!=8)
		return(E_FAIL);

	//create the various data from the header's numbers
	sMD2TexCoord* pTexCoords=new sMD2TexCoord[header.NumTexCoords];
	sMD2Face* pFaces=new sMD2Face[header.NumTriangles];
	sMD2Frame* pFrames=new sMD2Frame[header.NumFrames];

	//move to texture coordinents and read them in
	fseek(file,header.TexCoordOffset,SEEK_SET);
	fread(pTexCoords,sizeof(sMD2TexCoord),header.NumTexCoords,file);

	//move to the faces and read
	fseek(file,header.TriangleOffset,SEEK_SET);
	fread(pFaces,sizeof(sMD2Face),header.NumTriangles,file);

	//move to the frames
	fseek(file,header.FrameOffset,SEEK_SET);

	//loop through the frames and read in each one
	for(int i=0;i<header.NumFrames;i++)
	{
		//allocate new memory for the frame's vertices
		pFrames[i].pVertices=new sMD2Triangle[header.NumVertices];

		//fread in the frame of animation
		sMD2AliasFrame* frame=(sMD2AliasFrame*)buffer;
		fread(frame,1,header.FrameSize,file);

		//copy the vertex positions into the frames
		for(int j=0;j<header.NumVertices;j++)
		{
			pFrames[i].pVertices[j].Vertex[0]=frame->AliasVertices[j].Vertex[0]*frame->Scale[0]+frame->Translation[0];
			pFrames[i].pVertices[j].Vertex[1]=frame->AliasVertices[j].Vertex[2]*frame->Scale[2]+frame->Translation[2];
			pFrames[i].pVertices[j].Vertex[2]=frame->AliasVertices[j].Vertex[1]*frame->Scale[1]+frame->Translation[1];
		}
	}

	//create the member frames
	m_pFrames=new sMD2ModelFrame[header.NumFrames];

	//loop through and copy the vertices
	for(i=0;i<header.NumFrames;i++)
	{
		//create the vertices
		m_pFrames[i].pVertices=new MD2MODELVERTEX[header.NumTriangles*3];
		
		//now loop through and copy the vertices
		for(int j=0;j<header.NumTriangles*3;j++)
		{
			//copy the data
			MD2MODELVERTEX* pVerts=m_pFrames[i].pVertices;
			sMD2Face* pFace=&pFaces[j/3];
			pVerts[j].pos.x=pFrames[i].pVertices[pFace->VertexIndices[j%3]].Vertex[0];
			pVerts[j].pos.y=pFrames[i].pVertices[pFace->VertexIndices[j%3]].Vertex[1];
			pVerts[j].pos.z=pFrames[i].pVertices[pFace->VertexIndices[j%3]].Vertex[2];
			pVerts[j].dif=D3DCOLOR_ARGB(255,255,255,255);
			pVerts[j].tu=pTexCoords[pFace->TextureIndices[j%3]].tu;
			pVerts[j].tv=pTexCoords[pFace->TextureIndices[j%3]].tv;
		}
	}

	//set number of vertices
	m_NumVertices=header.NumTriangles*3;

	//delete the data structures before
	delete[] pTexCoords;
	delete[] pFrames;
	delete[] pFaces;
	
	//create the vertex buffer
	if(FAILED(hr=pDevice->CreateVertexBuffer(sizeof(MD2MODELVERTEX)*header.NumTriangles*3,
								D3DUSAGE_DYNAMIC|D3DUSAGE_WRITEONLY,
								D3DFVF_MD2MODELVERTEX,
								D3DPOOL_DEFAULT,
								&m_VertexBuffer)))
		return(hr);

	if(FAILED(hr=Update(0,0,0.0f,pDevice)))
		return(hr);
								

	//close the file
	fclose(file);

	//return success
	return(S_OK);
}
//releases the memory for this class
HRESULT CW3DMD2Model::Destroy()
{
	//loop through the frames if any and destroy them
	if(m_pFrames)
	{
		for(int i=0;i<m_NumFrames;i++)
		{
			delete[] m_pFrames[i].pVertices;
		}
		//delete the frames itself
		delete[] m_pFrames;
		m_pFrames=0;
	}
	m_NumFrames=0;

	//free the vertex buffer
	if(m_VertexBuffer)
	{
		m_VertexBuffer->Release();
		m_VertexBuffer=0;
	}

	//return success
	return(S_OK);
}
//render the model
//takes two keyframes and an interpolation between them
HRESULT CW3DMD2Model::Render(IDirect3DDevice8* pDevice)
{
	//render the model
	if(FAILED(hr=pDevice->SetVertexShader(D3DFVF_MD2MODELVERTEX)))
		return(hr);
	if(FAILED(hr=pDevice->SetStreamSource(0,m_VertexBuffer,sizeof(MD2MODELVERTEX))))
		return(hr);
	if(FAILED(hr=pDevice->DrawPrimitive(D3DPT_TRIANGLELIST,0,m_NumVertices/3)))
		return(hr);

	//return success
	return(S_OK);
}
//updates the model with the new frames of animation
//it interpolates between the keyframes
HRESULT CW3DMD2Model::Update(int Frame1,int Frame2,float interpolation,IDirect3DDevice8* pDevice)
{
	//lock the buffer first
	MD2MODELVERTEX* Data;
	if(FAILED(hr=m_VertexBuffer->Lock(0,0,(BYTE**)&Data,D3DLOCK_DISCARD)))
		return(hr);

	//loop through the vertices and find the interpolated the vertex
	for(int i=0;i<m_NumVertices;i++)
	{
		Data[i].tu=m_pFrames[Frame1].pVertices[i].tu;
		Data[i].tv=m_pFrames[Frame1].pVertices[i].tv;
		Data[i].dif=m_pFrames[Frame1].pVertices[i].dif;
		//linear interpolate between the positions
		D3DXVec3Lerp(&Data[i].pos,//out
					&m_pFrames[Frame1].pVertices[i].pos,
					&m_pFrames[Frame2].pVertices[i].pos,
					interpolation);
	}

	//unlock the buffer sinse we're finished
	if(FAILED(hr=m_VertexBuffer->Unlock()))
		return(hr);

	//return success
	return(S_OK);
}