//CW3DTrees.h
//Brent Gunning
//Copyright 2002 All Rights Reserved

//based on a loaded milkshape model, it creates trees to render.
//the milkshape model contains vertices which tell the points at which
//trees should be located. then we create billboards for the trees and
//setup particle systems for the leaves. finally, when rendering, we 
//use frustrum culling to eliminate the unneeded trees. 

//Note: The class uses triangles to draw the billboards not quads. 
//This means that the textures must have the tree picture in the top
//right corner

//define the file
#ifndef CW3DTREES_H
#define CW3DTREES_H

//include files
#include "d3d8.h"
#include "d3dx8.h"
#include "../../w3d engine/headers/VertexFormats.h"
#include "../../w3d engine/headers/CW3DParticleSystem.h"
#include "../../w3d engine/headers/MilkshapeLoader.h"
#include "../../w3d engine/headers/CW3DFrustum.h"
#include "../../Options.h"

//define for the tree vertex
#define TREEVERTEX VERTEX_XYZ_NRM_DIF_TEX
#define D3DFVF_TREEVERTEX D3DFVF_VERTEX_XYZ_NRM_DIF_TEX

//structure for a single tree
struct sTree
{
	TREEVERTEX Vertices[3];
	TREEVERTEX TransformedVertices[3];
	D3DXVECTOR3 vecPos;
	CW3DParticleSystem psystem;
	D3DXMATRIX matTransform;
	bool bVisible;
};

//the class which loads the renders the trees. uses frustrum culling
class CW3DTrees
{
public:
	//constructor/deconstructor
	//releases memory and sets defaults
	CW3DTrees();
	~CW3DTrees();

	//destroys the class's memory
	//this is called by the deconstructor
	HRESULT Destroy();

	//initializes the trees. loads them up and prepares to render
	HRESULT Initialize(IDirect3DDevice8* pDevice);

	//renders the trees. it takes a pointer to the viewing
	//frustrum so that it can figure out what is currently 
	//in the scene. it also takes the device for d3d work
	HRESULT Render(D3DXVECTOR3 vecCamRot,CW3DFrustum* pFrustum,IDirect3DDevice8* pDevice);
protected:
	//the trees
	sTree* m_pTrees;
	int m_NumTrees;

	//a vertex buffer to hold the trees
	IDirect3DVertexBuffer8* m_VertexBuffer;

	//the texture for the trees and the particles/leaves
	IDirect3DTexture8* m_TreeTexture;
	IDirect3DTexture8* m_LeafTexture;

	//error variable
	HRESULT hr;
};
#endif