//CW3DWorld.h
//Brent Gunning
//Copyright 2002 All Rights Reserved

//defines the level and world data. This includes the terrain, water, trees, and
//other level-related objects. It does not include the main character. It loads
//and renders the level objects.

//define the file
#ifndef CW3DWORLD_H
#define CW3DWORLD_H

//include files
#include "d3d8.h"
#include "d3dx8.h"
#include "CW3DTrees.h"
#include "../../w3d engine/headers/CW3DSkyBox.h"
#include "../../w3d engine/headers/CW3DMilkshapeModel.h"
#include "../../w3d engine/headers/CW3DCamera.h"
#include "CW3DWater.h"
#include "../../Options.h"

//the main world class
//loads, renders, and destroys the demo level
class CW3DWorld
{
public:
	//constructor/deconstructor
	//sets default values and releases memory
	CW3DWorld();
	~CW3DWorld();

	//loads the initilaizes the world. It loads all the files needed and prepares it
	//to be rendered with small updates
	HRESULT Initialize(IDirect3DDevice8* pDevice);

	//renders the entire world. This includes terrain, water, and more. This is the main
	//function for rendering the world. it takes a camera pointer so it can setup
	//billboards and skyboxes and other stuff.
	HRESULT Render(IDirect3DDevice8* pDevice,CW3DCamera* pCamera);

	//releases the world's memory. It calls the sub destroy functions inside of the classes
	HRESULT Destroy();
protected:
	//the trees with leaves
	CW3DTrees m_Trees;
	//this stores the skybox that surrounds the user
	CW3DSkyBox m_SkyBox;
	//the actual level loaded from file
	CW3DMilkshapeModel m_Level;
	//the water/waves/ocean floor
	CW3DWater m_Water;
	//the volcano particle system
	CW3DParticleSystem m_Volcano;
	IDirect3DTexture8* m_VolcTex;
	//error variable to return values from d3d functions
	HRESULT hr;
};
#endif