//CW3DTrees.cpp
//Brent Gunning
//Copyright 2002 All Rights Reserved

//include header file
#include "../headers/CW3DTrees.h"

//constructor/deconstructor
//releases memory and sets defaults
CW3DTrees::CW3DTrees()
{
	//set default memory(0's)
	m_pTrees=0;
	m_NumTrees=0;
	m_VertexBuffer=0;
	m_TreeTexture=m_LeafTexture=0;
	hr=0;
}
CW3DTrees::~CW3DTrees()
{
	//destroy the class's memory
	Destroy();
}
//destroys the class's memory
//this is called by the deconstructor
HRESULT CW3DTrees::Destroy()
{
	//delete the trees
	if(m_pTrees)
	{
		for(int i=0;i<m_NumTrees;i++)
			m_pTrees[i].psystem.Destroy();
		delete[] m_pTrees;
		m_pTrees=0;
	}
	//release the vertex buffer
	if(m_VertexBuffer)
	{
		m_VertexBuffer->Release();
		m_VertexBuffer=0;
	}
	//release the textures
	if(m_TreeTexture)
	{
		m_TreeTexture->Release();
		m_TreeTexture=0;
	}
	if(m_LeafTexture)
	{
		m_LeafTexture->Release();
		m_LeafTexture=0;
	}
	//return success
	return(S_OK);
}
//initializes the trees. loads them up and prepares to render
HRESULT CW3DTrees::Initialize(IDirect3DDevice8* pDevice)
{
	//first destroy the model if it was already initialized
	Destroy();

	//first load the milkshape data into the milkshape data structure
	sMilkshapeData* pMilkshapeData=LoadMilkshapeModel(TREE_FILE);
	
	//next check if it loaded correctly by checking it the data is valid
	if(pMilkshapeData==0)
		return(E_FAIL);

	//create the trees from the number of vertices and set the number of trees
	//equal to the number of vertices. Each vertex in the model stands for one 
	//tree position
	m_NumTrees=(int)pMilkshapeData->NumVertices-1;
	m_pTrees=new sTree[m_NumTrees];

	//create a particle system structure to hold the particle data
	//for each particle system
	sParticleSysProps psp;
	psp.bRemakeParticles=true;
	psp.cEndColor=LEAF_ENDCOLOR;
	psp.cStartColor=LEAF_STARTCOLOR;
	psp.fWeight=LEAF_WEIGHT;
	psp.fWeightRand=LEAF_WEIGHT+0.0001f;
	psp.fXSize=LEAF_SIZE;
	psp.fYSize=LEAF_SIZE;
	psp.iLifetime=LEAF_LIFETIME;
	psp.iLifetimeRand=LEAF_LIFETIME/3;
	psp.iNumParticles=NUM_LEAVES;
	psp.vecInitVel=D3DXVECTOR3(0.0f,0.0f,0.0f);
	psp.vecPosRand=LEAF_STARTBOUNDING;
	psp.vecVelRand=D3DXVECTOR3(0.1f,0.1f,0.1f);
	psp.vecWind=WIND_DIRECTION;
	psp.vecInitPos=D3DXVECTOR3(0,0,0);

	for(int i=0;i<m_NumTrees;i++)
	{
		//set the position of the tree
		m_pTrees[i].vecPos=pMilkshapeData->pVertices[i].vecPos;

		//setup the particle system for the tree
		if(FAILED(hr=m_pTrees[i].psystem.Initialize(psp,pDevice)))
			return(hr);

		//setup the tree vertices
		//top left vertex
		m_pTrees[i].Vertices[0].pos=D3DXVECTOR3(-TREE_SIZE,TREE_SIZE,0);
		m_pTrees[i].Vertices[0].nrm=D3DXVECTOR3(0,0,-1);
		m_pTrees[i].Vertices[0].dif=D3DCOLOR_ARGB(255,255,255,255);
		m_pTrees[i].Vertices[0].tu=0;
		m_pTrees[i].Vertices[0].tv=0;
		//top right vertex
		m_pTrees[i].Vertices[1].pos=D3DXVECTOR3(TREE_SIZE,TREE_SIZE,0);
		m_pTrees[i].Vertices[1].nrm=D3DXVECTOR3(0,0,-1);
		m_pTrees[i].Vertices[1].dif=D3DCOLOR_ARGB(255,255,255,255);
		m_pTrees[i].Vertices[1].tu=1;
		m_pTrees[i].Vertices[1].tv=0;
		//bottom right vertex
		m_pTrees[i].Vertices[2].pos=D3DXVECTOR3(TREE_SIZE,-TREE_SIZE,0);
		m_pTrees[i].Vertices[2].nrm=D3DXVECTOR3(0,0,-1);
		m_pTrees[i].Vertices[2].dif=D3DCOLOR_ARGB(255,255,255,255);
		m_pTrees[i].Vertices[2].tu=1;
		m_pTrees[i].Vertices[2].tv=1;

		//copy the tree vertices into the transformed vertices
		m_pTrees[i].TransformedVertices[0]=m_pTrees[i].Vertices[0];
		m_pTrees[i].TransformedVertices[1]=m_pTrees[i].Vertices[1];
		m_pTrees[i].TransformedVertices[2]=m_pTrees[i].Vertices[2];

		//determine it's translation matrix
		D3DXMatrixIdentity(&m_pTrees[i].matTransform);
		D3DXMatrixTranslation(&m_pTrees[i].matTransform,m_pTrees[i].vecPos.x,m_pTrees[i].vecPos.y+4.0f*TREE_SIZE/5.0f,m_pTrees[i].vecPos.z);
	}

	//load the textures into the member variables using the d3dx helper functions
	//load texture
	if(FAILED(hr=D3DXCreateTextureFromFileEx(pDevice,LEAF_TEXTURE,300,300,0,0,D3DFMT_UNKNOWN,
		D3DPOOL_MANAGED,D3DX_DEFAULT,D3DX_DEFAULT,D3DCOLOR_ARGB(255,0,0,0),0,0,&m_LeafTexture)))
		return(hr);
	if(FAILED(hr=D3DXCreateTextureFromFileEx(pDevice,TREE_TEXTURE,300,300,0,0,D3DFMT_UNKNOWN,
		D3DPOOL_MANAGED,D3DX_DEFAULT,D3DX_DEFAULT,D3DCOLOR_ARGB(255,0,0,0),0,0,&m_TreeTexture)))
		return(hr);

	//create the vertex buffer. we set it to be dynamic because we'll be locking it and 
	//copying the vertices every frame into them. we also make it write only because
	//we'll only be writing to it, never reading
	if(FAILED(hr=pDevice->CreateVertexBuffer(sizeof(TREEVERTEX)*m_NumTrees*3,//size
												D3DUSAGE_DYNAMIC|D3DUSAGE_WRITEONLY,//usage
												D3DFVF_TREEVERTEX,
												D3DPOOL_DEFAULT,
												&m_VertexBuffer)))
			return(hr);

	//finally, delete the milkshape data that we loaded up before
	delete pMilkshapeData;

	//return success
	return(S_OK);
}
//renders the trees. it takes a pointer to the viewing
//frustrum so that it can figure out what is currently 
//in the scene. it also takes the device for d3d work
HRESULT CW3DTrees::Render(D3DXVECTOR3 vecCamRot,CW3DFrustum* pFrustum,IDirect3DDevice8* pDevice)
{
	//here we create the billboard matrix by rotating
	//the billboard to match the rotation of the camera
	D3DXMATRIX matRot,matRotTemp;
	D3DXMatrixRotationX(&matRot,vecCamRot.x);
	D3DXMatrixRotationY(&matRotTemp,vecCamRot.y);
	D3DXMatrixMultiply(&matRot,&matRot,&matRotTemp);
	D3DXMatrixRotationZ(&matRotTemp,vecCamRot.z);
	D3DXMatrixMultiply(&matRot,&matRot,&matRotTemp);
	
	//now we set the render states for both the tree and particles rendered. They both
	//need to eliminate the black space around the texture so what we do is tell d3d 
	//through alpha testing that we want them eliminated
	if(FAILED(hr=pDevice->SetRenderState(D3DRS_ALPHAREF,0)))
		return(hr);
    if(FAILED(hr=pDevice->SetRenderState(D3DRS_ALPHATESTENABLE,true)))
		return(hr);
    if(FAILED(hr=pDevice->SetRenderState(D3DRS_ALPHAFUNC,D3DCMP_GREATER)))
		return(hr);

	//determine the matrix needed for height values. this gets multiplied by
	//the tree transform to raise the height for rendering the particle leaves
	D3DXMATRIX matHeight;
	D3DXMatrixIdentity(&matHeight);
	D3DXMatrixTranslation(&matHeight,0,4.0f*TREE_SIZE/5.0f,0);

	//variable to rendering trees
	int NumRenderTrees=0;
	//copy the tree vertices into the global vertex buffer
	TREEVERTEX* Data;//lock the vertex buffer for copying
	if(FAILED(hr=m_VertexBuffer->Lock(0,0,(BYTE**)&Data,D3DLOCK_DISCARD)))
		return(hr);
	//loop and increment a variable telling how many trees to render
	for(int i=0;i<m_NumTrees;i++)
	{
		//check if the tree is visible. if it is, increment the tree render variable,
		//copy the data, and process and render the particle system for that tree
		if((m_pTrees[i].bVisible=pFrustum->IsSphereInFrustum(m_pTrees[i].vecPos.x,m_pTrees[i].vecPos.y,
											m_pTrees[i].vecPos.z,TREE_SIZE)==true))
		{
			//transform the vertices' positions
			D3DXMATRIX matTransform=matRot*m_pTrees[i].matTransform,matTemp;
			D3DXVec3TransformCoord(&m_pTrees[i].TransformedVertices[0].pos,&m_pTrees[i].Vertices[0].pos,&matTransform);
			D3DXVec3TransformCoord(&m_pTrees[i].TransformedVertices[1].pos,&m_pTrees[i].Vertices[1].pos,&matTransform);
			D3DXVec3TransformCoord(&m_pTrees[i].TransformedVertices[2].pos,&m_pTrees[i].Vertices[2].pos,&matTransform);
		
			//transform the normals
			D3DXVec3TransformNormal(&m_pTrees[i].TransformedVertices[0].nrm,&m_pTrees[i].Vertices[0].nrm,&matTransform);
			D3DXVec3TransformNormal(&m_pTrees[i].TransformedVertices[1].nrm,&m_pTrees[i].Vertices[1].nrm,&matTransform);
			D3DXVec3TransformNormal(&m_pTrees[i].TransformedVertices[2].nrm,&m_pTrees[i].Vertices[2].nrm,&matTransform);
			
			//copy the vertices into the buffer
			memcpy(&Data[NumRenderTrees*3+0],&m_pTrees[i].TransformedVertices[0],sizeof(TREEVERTEX));
			memcpy(&Data[NumRenderTrees*3+1],&m_pTrees[i].TransformedVertices[1],sizeof(TREEVERTEX));
			memcpy(&Data[NumRenderTrees*3+2],&m_pTrees[i].TransformedVertices[2],sizeof(TREEVERTEX));
		
			//increment the numbr of rendered trees variable
			NumRenderTrees++;
		}
	}
	//we're finished, unlock it now
	if(FAILED(hr=m_VertexBuffer->Unlock()))
		return(hr);

	//render the trees
	if(NumRenderTrees>0)
	{
		//the trees are all set to render. all that's left is setting the textue and rendering
		//the following code does just that
		if(FAILED(hr=pDevice->SetTexture(0,m_TreeTexture)))
			return(hr);
		if(FAILED(hr=pDevice->SetStreamSource(0,m_VertexBuffer,sizeof(TREEVERTEX))))
			return(hr);
		if(FAILED(hr=pDevice->SetVertexShader(D3DFVF_TREEVERTEX)))
			return(hr);

		//set the world matrix to the identity matrix
		D3DXMATRIX matWorld;
		D3DXMatrixIdentity(&matWorld);
		if(FAILED(hr=pDevice->SetTransform(D3DTS_WORLD,&matWorld)))
			return(hr);

		if(FAILED(hr=pDevice->DrawPrimitive(D3DPT_TRIANGLELIST,0,NumRenderTrees)))
			return(hr);

		//set the leaf texture for rendering the particle systems
		if(FAILED(hr=pDevice->SetTexture(0,m_LeafTexture)))
			return(hr);

		//loop through the number of trees and render their particle systems
		for(int i=0;i<m_NumTrees;i++)
		{
			if(m_pTrees[i].bVisible)
			{
				if(FAILED(hr=m_pTrees[i].psystem.Process()))
					return(hr);
				if(FAILED(hr=m_pTrees[i].psystem.Render(pDevice,matHeight*m_pTrees[i].matTransform)))
					return(hr);
			}
		}
	}

	//return success
	return(S_OK);
}